% *************************************************************************
%  Khan, Hashmat and Abeer Reza (2016), "House Prices and Government
%  Spending Shocks", Journal of Money, Credit and Banking
% 
%  Replication File for Figure 1
% *************************************************************************

clear all; 
clc;

ddata = [];

% Read in the data
% Data from 1966:04 to 2010:03. 
% --------------------------------------------
   
ddata = xlsread('GOVSHK_KR.xlsx');

% --------------------------------------------

% All data are log nominal per capita deflated by GDP deflator

G = log(ddata(:,1));                % Real per capita government Spending 
T_PGDP = (ddata(:,2));              % Real per capita Gov Tax + Transfer Receipts - Transfer Payments - Subsidies and interestrate payments (deflated by GDP deflator)
Y = log(ddata(:,3));                % Real per capita ouput
C = log(ddata(:,4));                % Real per capita consumption less housing & utilities 
FFED = ddata(:,5);                  % Effective Fed Rate
PGDP = ddata(:,6);                  % GDP deflator
HP_CenMed_PGDP = log(ddata(:,7));   % Real House Prices (Census median new single-family home sales) deflated by GDP deflator
FESPFGB = ddata(:,8);               % Auerbach Gorodnichenko FORECAST ERROR in government spending, max value from Green Book;


% ************************************************************************ 
% This is the data matrix. Modifying this will produce different Chol ordered
% figures. The two cases are defined as: 
% 1) Simple Cholesky, no News
% 2) Cholesky with NEWS: SPF forecast error ordered first. Shock to
% Forecast Errors

% Define House Prices: 
HPI = HP_CenMed_PGDP; 
% Define Gov Revenue: 
X = T_PGDP;

% Define the Real Fed Funds Rate: 
RFFED = zeros(size(FFED));
for tt = 1: size(FFED,1)-1;
RFFED(tt) = FFED(tt) - 100*((PGDP(tt+1)/PGDP(tt))^4-1); 
end; 

figure('Position', [100, 100, 1000, 1000]);

for CASE =1:2;

if CASE ==1;
    YY= [G X Y C HPI RFFED];
    YY = YY(1:end-1,:);
    plotindex = [1;3;4;5;6];
    shockindex = 1;
    ptitle =0;
elseif CASE ==2;
    YY= [FESPFGB G X Y C HPI RFFED];
    YY = YY(1:end-1,:);
    plotindex = [2;4;5;6;7];
    shockindex = 1;
    ptitle = 1;
end;
% ************************************************************************ 


% Preliminary transformation of the data

T = size(YY,1);
nvars = size(YY,2);
lags = 4;           % number of lags
nimp = 20;          % Horizon for impulse responses
const=ones(T,1);

EXOG = [const]; 
exogp = size(EXOG,2);


% Now we can put the lags of the LHS variables as usual:
XX = EXOG;
for i = 1:lags;
    XX = [XX lagn(YY,i)];
end

YY = YY(lags+1:end,:);
XX = XX(lags+1:end,:);


% Estiamtion of the VAR system

beta0 = olssvd(YY, XX);
e = YY - XX*beta0;
Sigmad = cov(e);
P = chol(Sigmad, 'lower');

% *****************************************************
% Monte Carlo run: 

ndraws = 1000;
sb1 = size(beta0,1);
sb2 = size(beta0,2);
np=size(beta0,1)*size(beta0,2);

Sigma_e=[];
Bet = [];
B_mc = [];
mc_res = [];
P_e = [];

for i=1:ndraws
     PS = real(sqrtm(inv(T*Sigmad))); 
     u = randn(nvars,T);
     Sigma_e(:,:,i) = inv(PS*u*u'*PS');
     [uu s vv]=svd((kron(Sigma_e(:,:,i),inv(XX'*XX))));
     Bet(:,i)=reshape(beta0,np,1)+real(uu*sqrt(s)*vv')*randn(np,1);
     B_mc(:,:,i) =reshape(Bet(:,i),sb1, sb2); 
     P_e(:,:,i) = chol(Sigma_e(:,:,i), 'lower');
disp('Monte Carlo Draw') 
i 
end


% *****************************************************


% choleski decomposition for Original Beta OLS
    beta = beta0(exogp+1:end,:);
    beta = beta';
    [m1,m2]=size(beta);
    BB = zeros(m1*lags,m2);
    for ll = 1:(lags-1)
        BB(ll*m1+1:(ll+1)*m1, (ll-1)*m1+1:ll*m1) = eye(m1);
    end;
    BB(1:m1,:) = beta;

wimpu=[];
cimpu=[];

for j=1:nimp
        %% (non-orthogonal) Wold impulses
        if j==1
            wimpu(:,:,j)=eye(size(BB,1),size(BB,2));
        elseif j>1
            wimpu(:,:,j)=BB^(j-1);
        end
        %% (orthogonal) Choleski impulses
        cimpu(:,:,j)=wimpu(1:nvars,1:nvars,j)*P;
end

% Calculate Choleski Decomposed IRF's for Gov Shock
XAX = zeros(1,nimp);
STDV = sqrtm(Sigmad);
G_IMPACT = exp(squeeze(cimpu(1,1,1)));


% ************************************************************************
% Add Monte Carlo Bands
wimpu=[];
cimpu=[];
basicimp = [];

for ii = 1:ndraws
    beta = B_mc(exogp+1:end,:,ii);
    beta = beta';
    [m1,m2]=size(beta);
    BB = zeros(m1*lags,m2);
    for ll = 1:(lags-1)
        BB(ll*m1+1:(ll+1)*m1, (ll-1)*m1+1:ll*m1) = eye(m1);
    end;
    BB(1:m1,:) = beta;
for j=1:nimp
        %% (non-orthogonal) Wold impulses
        if j==1
            wimpu(:,:,j)=eye(size(BB,1),size(BB,2));
        elseif j>1
            wimpu(:,:,j)=BB^(j-1);
        end
        %% (orthogonal) Choleski impulses
        cimpu(:,:,j)=wimpu(1:nvars,1:nvars,j)*P_e(:,:,ii);
end

basicimp(:,:,:,ii) = cimpu(:,:,:);
end;

bi = sort(basicimp,4);


% Calculate Choleski Decomposed IRF's for Gov Shock
STDV = sqrtm(Sigmad);


for jj=1:5
    indx = plotindex(jj);
    
    me_irf = squeeze(bi(indx,shockindex,:,fix(ndraws*.50)))/(STDV(indx,indx));
    up_irf = squeeze(bi(indx,shockindex,:,fix(ndraws*.84)))/(STDV(indx,indx));
    lo_irf = squeeze(bi(indx,shockindex,:,fix(ndraws*.16)))/(STDV(indx,indx));
%  ****************************************************************

            
    subplot(5,2,jj+ (jj-1) + (CASE-1)),plot(1:nimp, me_irf,'-k',1:nimp, up_irf,'--b', 1:nimp, lo_irf, '--b', 1:nimp, XAX','-r','Linewidth',1.25),axis tight; 
    if CASE ==1;
    if jj==1;
        ylabel('Gov Spending');
        title('Cholesky');
    end;
    if jj == 2;
        ylabel('Output');
    end; 
    if jj ==3; 
        ylabel('Consumption');
    end; 
    if jj == 4; 
        ylabel('House Prices');
    end;
    if jj == 5; 
        ylabel('Real Fed Funds Rate');
    end;
    end; 
    if CASE == 2;
        if jj ==1;
            title('Forecast Error');
        end;
    elseif CASE == 3;
        if jj ==1;
            title('War Dates');
        end;
    end;
    
end

end;

save2pdf('Figure1');
 


